@@ -8,7 +8,7 @@ Locale::CLDR - A Module to create locale objects with localisation data from the
 
 =head1 VERSION
 
-Version 0.26.7
+Version 0.26.8
 
 =head1 SYNOPSIS
 
@@ -39,7 +39,7 @@ or
 
 use v5.10;
 use version;
-our $VERSION = version->declare('v0.26.7');
+our $VERSION = version->declare('v0.26.8');
 
 use open ':encoding(utf8)';
 use utf8;
@@ -2441,16 +2441,23 @@ sub _unit_compound {
 	$type //= 'long';
 	
 	my $dividend = $self->unit($number, $dividend_what, $type);
-	my $divisor = $self->unit(1, $divisor_what, $type);
+	my $divisor = $self->_unit_per($divisor_what, $type);
+	if ($divisor) {
+		my $format = $divisor;
+		$format =~ s/\{0\}/$dividend/;
+		return $format;
+	}
+	
+	$divisor = $self->unit(1, $divisor_what, $type);
 	
 	my $one = $self->format_number(1);
 	$divisor =~ s/\s*$one\s*//;
-
+	
 	my @bundles = $self->_find_bundle('units');
 	my $format;
 	foreach my $bundle (@bundles) {
-		if (exists $bundle->units()->{$type}{per}{1}) {
-			$format = $bundle->units()->{$type}{per}{1};
+		if (exists $bundle->units()->{$type}{per}{''}) {
+			$format = $bundle->units()->{$type}{per}{''};
 			last;
 		}
 	}
@@ -2476,6 +2483,83 @@ sub _unit_compound {
 	return $format;
 }
 
+=item unit_name($unit_identifier)
+
+This method returns the localised name of the unit
+
+=cut
+
+sub unit_name {
+    my ($self, $what) = @_;
+	
+	my @bundles = $self->_find_bundle('units');
+	my $name;
+	foreach my $bundle (@bundles) {
+		if (exists $bundle->units()->{long}{$what}{name}) {
+			return $bundle->units()->{long}{$what}{name};
+		}
+	}
+	
+	# Check for aliases
+	my $type = 'long';
+	my @aliases = $self->_find_bundle('unit_alias');
+	foreach my $alias (@aliases) {
+		$type = $alias->unit_alias()->{$type};
+		next unless $type;
+		foreach my $bundle (@bundles) {
+			if (exists $bundle->units()->{$type}{$what}{name}) {
+				return $bundle->units()->{$type}{$what}{name};
+			}
+		}
+	}
+	
+	return '';
+}
+
+sub _unit_per {
+    my ($self, $what, $type) = @_;
+	
+	my @bundles = $self->_find_bundle('units');
+	my $name;
+	foreach my $bundle (@bundles) {
+		if (exists $bundle->units()->{$type}{$what}{per}) {
+			return $bundle->units()->{$type}{$what}{per};
+		}
+	}
+	
+	# Check for aliases
+	my @aliases = $self->_find_bundle('unit_alias');
+	foreach my $alias (@aliases) {
+		$type = $alias->unit_alias()->{$type};
+		next unless $type;
+		foreach my $bundle (@bundles) {
+			if (exists $bundle->units()->{$type}{$what}{per}) {
+				return $bundle->units()->{$type}{$what}{per};
+			}
+		}
+	}
+	
+	return '';
+}
+
+sub _get_time_separator {
+	my $self = shift;
+
+	my @number_symbols_bundles = $self->_find_bundle('number_symbols');
+	my $symbols_type = $self->default_numbering_system;
+	
+	foreach my $bundle (@number_symbols_bundles) {	
+		if (exists $bundle->number_symbols()->{$symbols_type}{alias}) {
+			$symbols_type = $bundle->number_symbols()->{$symbols_type}{alias};
+			redo;
+		}
+		
+		return $bundle->number_symbols()->{$symbols_type}{timeSeparator}
+			if exists $bundle->number_symbols()->{$symbols_type}{timeSeparator};
+	}
+	return ':';
+}
+
 =item duration_unit($format, @data)
 
 This method formats a duration. The format must be one of
@@ -2497,6 +2581,10 @@ sub duration_unit {
 		$num_format = '00' if $parsed =~ s/$entry/$self->format_number(shift(@data), $num_format)/e;
 	}
 	
+	my $time_separator = $self->_get_time_separator;
+	
+	$parsed =~ s/:/$time_separator/g;
+	
 	return $parsed;
 }
 
@@ -3291,7 +3379,11 @@ sub _build_any_time_format {
 			}
 			
 			my $result = $time_formats->{$default_calendar}{$width};
-			return $result if $result;
+			if ($result) {
+				my $time_separator = $self->_get_time_separator;
+				$result =~ s/:/$time_separator/g;
+				return $result;
+			}
 		}
 		if ($default_calendar ne 'gregorian') {
 			$default_calendar = 'gregorian';
